#' @title Plot the repeated measurements
#'
#' @description Plot both the distribution of the measurements
#' and the quantile-quantile plot. The theoretical distribution for the
#' quantile-quantile plot is the normal distribution.
#'
#' @param x
#' An object of class "RM_Evaluation"
#'
#' @param plot_type
#' Character string. If plot_type matches "dotplot" then the distribution of
#' the measurements is plotted as a dotplot. Otherwise,
#' it is plotted as a histogram.
#'
#' @param ...
#' Required for compatibility with generic definition
#'
#' @details
#' A dotplot
#' is better for a small number of measurements (say, less than
#' 150), whereas a histogram is better for a large number (say, greater than
#' 250).
#'
#' @examples
#' oEvaluation <- RM_Evaluation(ExampleData, "Example data (no units)")
#' plot(oEvaluation)
#'
#' @export
#'
plot.RM_Evaluation <- function(x, plot_type = "dotplot", ...) {

  df <- tibble::tibble(x = sort(x$measurements),
                       p = stats::ppoints(length(x)),
                       y = stats::qnorm(p, mean = mean(x), sd = stats::sd(x)))

  if(plot_type == "dotplot"){
    p1 <- ggplot2::ggplot() +
      ggplot2::geom_dotplot(ggplot2::aes(x = x), data = df,
                   method = "histodot") +
      ggplot2::scale_x_continuous(name = x$axisLabel) +
      ggplot2::scale_y_continuous(name = "", breaks = NULL) +
      ggplot2::ggtitle("A.") +
      ggplot2::theme_bw() +
      ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0,face = "bold", size = 10)) +
      ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank())
  } else {
    p1 <- ggplot2::ggplot() +
      ggplot2::geom_histogram(ggplot2::aes(x = x, y = ..density..),
                     data = df,
                     fill = "gray55", color = "white") +
      ggplot2::scale_x_continuous(name = x$axisLabel) +
      ggplot2::scale_y_continuous(name = "Density") +
      ggplot2::ggtitle("A.") +
      ggplot2::theme_bw() +
      ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0,face = "bold", size = 10)) +
      ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank())
  }


  limits <- range(c(df$x, df$y))

  p2 <- ggplot2::ggplot(df) +
    ggplot2::geom_abline(slope = 1, color = "red") +
    ggplot2::geom_point(ggplot2::aes(x = x, y = y), color = "black") +
    ggplot2::scale_x_continuous(name = paste0("Quantile of ", x$axisLabel), limits = limits) +
    ggplot2::scale_y_continuous(name = "Quantile of normal distribution", limits = limits) +
    ggplot2::coord_equal() +
    ggplot2::ggtitle("B.") +
    ggplot2::theme_bw() +
    ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0,face = "bold", size = 10)) +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank())

    grid::grid.newpage()
    grid::pushViewport(grid::viewport(layout=grid::grid.layout(1,2)))
    print(p1, vp=grid::viewport(layout.pos.row=1, layout.pos.col=1))
    print(p2, vp=grid::viewport(layout.pos.row=1, layout.pos.col=2))
}


#' @title Summarize the repeated measurements
#'
#' @description Print summary statistics of the repeated measurements
#' and print the p-value for the Shapiro-Wilks test of normality.
#'
#' @param object
#' An object of class "RM_Evaluation"
#'
#' @param ...
#' Required for compatibility with generic definition
#'
#' @examples
#' oEvaluation <- RM_Evaluation(ExampleData, "Example data (no units)")
#' summary(oEvaluation)
#'
#' @export
#'
summary.RM_Evaluation <- function(object, ...) {

  cat(sprintf( "###############################################################\n"))
  cat(sprintf("Summary\n"))
  cat(sprintf("Number of measurements: %g\n", length(object$measurements)))
  cat(sprintf("Range: %g   %g\n", object$theRange[1], object$theRange[2]))
  cat(sprintf("Mean: %g\n", object$theMean))
  cat(sprintf("Standard deviation: %g\n", object$stdDev))
  cat(sprintf("Median: %g\n", object$theMedian))
  cat(sprintf("IQR: %g\n", object$theIQR))
  cat(sprintf("P-value for the Shapiro-Wilks test of normality: %g\n",
               object$p_value ))
  cat(sprintf( "###############################################################\n"))

}


#' @title Constructor of class that evaluates repeated measurements.
#'
#' @description
#' This constructor calculates some quantities that are needed to evaluate
#' repeated measurements.
#'
#' @param measurements
#' R-Language vector containing the repeated measurements.
#'
#' @param axisLabel
#' Character string describing the measurements. This string is used as
#' an label in the plots.
#'
#' @return A list with the following components is returned.
#' @return \item{measurements}{Argument measurements.}
#' @return \item{axisLabel}{Argument axisLabel.}
#' @return \item{theRange}{Range of the measurements.}
#' @return \item{theMean}{Mean of the measurements.}
#' @return \item{stdDev}{Standard deviation of the measurements.}
#' @return \item{theMedian}{Median of the measurements.}
#' @return \item{theIQR}{Interquartile range of the measurements.}
#' @return \item{p_value}{P-value from the Shapiro-Wilk test of normality.}
#' @return \item{call}{Function call.}
#'
#' @examples
#'
#' oEvaluation <- RM_Evaluation(ExampleData, "Example data (no units)")
#'
#' @export
#'
RM_Evaluation <- function(measurements, axisLabel) {

  rval <- list(measurements = measurements,
               axisLabel = axisLabel,
               theRange = range(measurements),
               theMean = mean(measurements),
               stdDev = stats::sd(measurements),
               theMedian = stats::median(measurements),
               theIQR = stats::IQR(measurements),
               p_value = stats::shapiro.test(measurements)$p.value,
               call = sys.call())

  class(rval) <- "RM_Evaluation"

  return(rval)
}
